@@ -32,6 +32,12 @@ import ast
 
 import turtle
 
+try:
+    import optimism
+    OPTIMISTIC = True
+except Exception:
+    OPTIMISTIC = False
+
 from . import load
 from . import mast
 from . import context_utils
@@ -122,6 +128,7 @@ def create_module_import_payload(
         """
         filename = context_utils.extract(context, "filename")
         file_path = context_utils.extract(context, "file_path")
+        file_path = os.path.abspath(file_path)
         full_name = name_prefix + filename
         tinfo = context_utils.extract(context, "task_info")
         spec = tinfo["specification"]
@@ -152,6 +159,13 @@ def create_module_import_payload(
             sandbox_links=load.link_mapping(spec),
             on_disk=file_path
         )
+
+        # Attempt to release any input capturing set up by optimism calls
+        # in the file we just ran:
+        if OPTIMISTIC:
+            optimism.restoreOutput()
+            optimism.restoreInput()
+
         result = {
             "original_source": original_source,
             "source": fixed,
@@ -808,12 +822,43 @@ def with_fake_input(payload, inputs, extra_policy="error"):
         reset_input()
         builtins.input = mock_input
 
-        # Run the payload
-        result = payload(context)
+        # If we're dealing with optimism, it's mechanism for providing
+        # input (replacing stdin) won't work with our mock_input
+        # function! So we need to patch that up, and let
+        # optimism-specified input override mock input until
+        # restoreInput is called.
+        if OPTIMISTIC:
+            original_provideInput = optimism.provideInput
 
-        # Re-enable `input`
-        builtins.input = original_input
-        reset_input()
+            def provideInputNoMock(*args, **kwargs):
+                """
+                Disables mock input and calls original provideInput.
+                """
+                builtins.input = original_input
+                original_provideInput(*args, **kwargs)
+
+            original_restoreInput = optimism.restoreInput
+
+            def restoreInputToMock(*args, **kwargs):
+                builtins.input = mock_input
+                original_restoreInput(*args, **kwargs)
+
+            optimism.provideInput = provideInputNoMock
+            optimism.restoreInput = restoreInputToMock
+
+        # Run the payload
+        try:
+            result = payload(context)
+        finally:
+            # Re-enable `input`
+            builtins.input = original_input
+            reset_input()
+            reset_input()
+
+            # Reset optimism input mechanisms
+            if OPTIMISTIC:
+                optimism.provideInput = original_provideInput
+                optimism.restoreInput = original_restoreInput
 
         # Add "inputs" and "input_policy" context slots to the result
         result["inputs"] = inputs
