@@ -24,13 +24,15 @@ from . import render
 # Setup #
 #-------#
 
-
-def set_specs_dir(directory):
+def setup(specs_dir, sandbox_dir):
     """
-    Sets the specifications directory.
+    Sets the specifications and sandbox directories.
     """
-    global SPECS_DIR
-    SPECS_DIR = directory
+    global SPECS_DIR, SANDBOX_DIR
+    SPECS_DIR = specs_dir
+    SANDBOX_DIR = sandbox_dir
+    # Ensure sandboxes directory exists
+    os.makedirs(SANDBOX_DIR, exist_ok=True)
 
 
 #------------------#
@@ -234,7 +236,10 @@ def create_module_in_sandbox(
     code was parsed from, and will be used to provide a __file__
     variable while the code runs.
     """
-    with tempfile.TemporaryDirectory(suffix="__tmp") as tmpdir:
+    with tempfile.TemporaryDirectory(
+        suffix="__tmp",
+        dir=SANDBOX_DIR
+    ) as tmpdir:
         # Create symlinks
         if sandbox_links is not None:
             for filepath in sandbox_links:
