@@ -4,6 +4,9 @@ Logging support.
 logging.py
 """
 
+# For Python2 cross-compatibility
+from __future__ import print_function
+
 import sys, traceback
 
 LOG_TO = sys.stdout
