@@ -188,7 +188,13 @@ def render_markdown(md_source):
     except ModuleNotFound_or_Import_Error:
         x = 'extra'
 
-    return markdown.markdown(md_source, extensions=[x])
+    try:
+        # Try to render with extensions
+        return markdown.markdown(md_source, extensions=[x])
+    except Exception:
+        # Backup: try without extensions, since in some cases they don't
+        # work I guess :(
+        return markdown.markdown(md_source, extensions=[])
 
 
 #-----------------------------#
