@@ -12,7 +12,7 @@ import subprocess
 import pytest
 import importlib_resources
 
-from .. import report
+from .. import render
 from .._version import __version__ as potluck_version
 
 # Where to import potluck from so that we're testing the same potluck...
@@ -38,6 +38,18 @@ RUBRIC_EXPECTS = {
         "<code>ellipseArea</code> returns the correct result",
     ],
     "freedomTest": [ "<title>freedomTest Rubric</title>" ],
+    "snippetsTest": [
+        "<title>snippetsTest Rubric</title>",
+        "<h1>Rubric for snippetsTest</h1>",
+        "Product Requirements",
+        "Core goals",
+        "processData",
+        "returns the correct result",
+        "Behavior Requirements",
+        "Extra goals",
+        "process.py",
+        "exhibits the correct behavior"
+    ],
 }
 
 # Expectations about reports
@@ -61,9 +73,15 @@ REPORT_EXPECTS = {
     "freedomTest": {
         "perfect": { "evaluation": "excellent" },
         "imperfect": { "evaluation": "partially complete" },
+    },
+    "snippetsTest": {
+        "perfect": { "evaluation": "excellent" },
+        "imperfect": { "evaluation": "incomplete" },
     }
 }
 
+# TODO: Expectations for instructions and for snippets!
+
 
 @pytest.fixture(
     params=[
@@ -71,7 +89,8 @@ REPORT_EXPECTS = {
         "debugTest",
         "interactiveTest",
         "sceneTest",
-        "freedomTest"
+        "freedomTest",
+        "snippetsTest"
     ]
 )
 def taskid(request):
@@ -149,8 +168,8 @@ def check_log_is_clean(logfile):
     assert log.splitlines()[0] == (
         f"This is potluck version {potluck_version}"
     )
-    assert report.ERROR_MSG not in log
-    assert report.DONE_MSG in log
+    assert render.ERROR_MSG not in log
+    assert render.DONE_MSG in log
 
 
 def test_rubric_creation(in_evaldir, taskid, logfile, rubricfile):
