@@ -241,6 +241,18 @@ def time_spent_key(course, semester, username, pset, phase, task):
     )
 
 
+def evaluation_key(course, semester, username, pset, phase, task):
+    """
+    The Redis key for the custom evaluation info for a
+    class/semester/user/pset/phase/task.
+    """
+    return redis_key(
+        ':'.join(
+            [course, semester, "eval", username, pset, phase, task]
+        )
+    )
+
+
 #----------------#
 # Roster loading #
 #----------------#
@@ -861,12 +873,17 @@ def fetch_time_spent(course, semester, username, phase, psid, taskid):
     if response is None or len(response) != 3 or response[0] is None:
         return None
 
+    try:
+        spent = float(response[1])
+    except ValueError:
+        spent = response[1]
+
     result = {
         "phase": phase,
         "psid": psid,
         "taskid": taskid,
         "updated_at": response[0],
-        "time_spent": response[1]
+        "time_spent": spent
     }
 
     if response[2] is not None:
@@ -974,6 +991,168 @@ def record_time_spent(
         )
 
 
+def fetch_evaluation(course, semester, username, phase, psid, taskid):
+    """
+    Fetches the manual evaluation information for the given
+    user/phase/pset/task. The result will be a dictionary with the
+    following keys:
+
+    - "phase": The phase (a string).
+    - "psid": The problem-set ID (a string).
+    - "taskid": The task ID (a string).
+    - "updated_at": A timestring (see `potluck.time_utils.timestring`)
+        indicating when the information was last updated.
+    - "notes": The markdown source string for custom notes.
+    - "override": A numerical score that overrides the automatic
+        evaluation. Will be an empty string if there is no override to
+        apply.
+
+    Returns None instead of a dictionary if there is no information for
+    that user/pset/task yet, or if an error is encountered while trying
+    to access that information.
+    """
+    # Redis key to use
+    key = evaluation_key(
+        course,
+        semester,
+        username,
+        psid,
+        phase,
+        taskid
+    )
+
+    try:
+        response = _REDIS.hmget(
+            key,
+            "updated_at",
+            "notes",
+            "override"
+        )
+    except Exception:
+        flask.flash("Error fetching evaluation info.")
+        tb = potluck.html_tools.string_traceback()
+        print(
+            "Failed to fetch evaluation info at '{}':\n{}".format(
+                key,
+                tb
+            ),
+            file=sys.stderr
+        )
+        return None
+
+    # Some kind of non-exception error during access, or key is missing
+    if response is None or len(response) != 3 or response[0] is None:
+        return None
+
+    try:
+        override = float(response[2])
+    except ValueError:
+        override = response[2]
+
+    result = {
+        "phase": phase,
+        "psid": psid,
+        "taskid": taskid,
+        "updated_at": response[0],
+        "notes": response[1],
+        "override": override,
+    }
+
+    return result
+
+
+def set_evaluation(
+    course,
+    semester,
+    username,
+    phase,
+    psid,
+    taskid,
+    notes,
+    override=""
+):
+    """
+    Updates the custom evaluation info for a particular task submitted by
+    a particular use for a certain phase of a specific pset (in a
+    course/semester).
+
+    Completely erases the previous custom evaluation info.
+
+    The notes argument must be a string, and will be treated as Markdown
+    and converted to HTML when being displayed to the user. It will be
+    displayed on a feedback page and it can thus link to rubric items or
+    snippets by their IDs if you want to get fancy.
+
+    The override argument defaults to an empty string, which is how to
+    indicate that no override should be applied. Otherwise, it should be
+    a floating-point number or integer between 0 and 100; it will be
+    stored as a float if convertible.
+
+    Returns True if it succeeds or False if it encounters some sort of
+    error.
+    """
+    # Redis key to use
+    key = evaluation_key(
+        course,
+        semester,
+        username,
+        psid,
+        phase,
+        taskid
+    )
+
+    # Generate a timestamp for the info
+    timestring = potluck.time_utils.timestring()
+
+    # Convert to a number if we can
+    if override != "":
+        try:
+            override = float(override)
+            if 0 < override < 1:
+                flask.flash(
+                    (
+                        "Warning: you entered '{}' as the grade"
+                        " override, but scores should be specified out"
+                        " of 100, not out of 1! The override has been"
+                        " set as-given but you may want to update it."
+                    ).format(override)
+                )
+        except Exception:
+            flask.flash(
+                (
+                    "Warning: you entered '{}' as the grade override,"
+                    " but grade overrides should be numbers between 0"
+                    " and 100. The override has been set as-given, but"
+                    " you may want to update it."
+                ).format(override)
+            )
+
+    # Here's the info we store
+    info = {
+        "updated_at": timestring,
+        "notes": notes,
+        "override": override
+    }
+
+    try:
+        success = _REDIS.hmset(key, info)
+        if success is not True:
+            raise ValueError("Redis result indicated failure.")
+    except Exception:
+        flask.flash("Failed to write evaluation info!")
+        tb = potluck.html_tools.string_traceback()
+        print(
+            "Failed to write evaluation info at '{}':\n{}".format(
+                key,
+                tb
+            ),
+            file=sys.stderr
+        )
+        return False
+
+    return True
+
+
 def default_feedback_summary():
     """
     Returns a default summary object. The summary is a pared-down version
@@ -1043,15 +1222,18 @@ def get_feedback(
     given phase/pset/task. Instead of caching these values (which would
     be expensive memory-wise over time) we hit the disk every time.
 
-    Returns the string "missing" if the relevant feedback file does not
-    exist, or None if some kind of error occurs trying to access the
-    file.
+    Returns a dictionary with at least a 'status' entry. This will be
+    'ok' if the report was read successfully, or 'missing' if the report
+    file could not be read or did not exist. If the status is
+    'missing', a 'log' entry will be present with the contents of the
+    associated log file, or the string 'missing' if that log file could
+    also not be read.
 
     Weird stuff could happen if the file is being written as we make the
     request. Typically a second attempt should not re-encounter such an
     error.
     """
-    result = None
+    result = { "status": "unknown" }
     ts, log_file, report_file, status = get_inflight(
         course,
         semester,
@@ -1061,20 +1243,21 @@ def get_feedback(
         taskid
     )
     if ts is None: # No submission
-        result = "missing"
+        result["status"] = "missing"
     elif ts == "error": # Failed to read inflight file
         flask.flash(
             "Failed to fetch in-flight info; please refresh the page."
         )
-        result = "missing"
+        result["status"] = "missing"
 
-    if result != "missing":
+    if result["status"] != "missing":
         try:
             if not os.path.exists(report_file):
-                result = "missing"
+                result["status"] = "missing"
             else:
                 with open(report_file, 'r') as fin:
                     result = json.load(fin)
+                    result["status"] = "ok"
         except Exception:
             flask.flash("Failed to read feedback file.")
             tb = potluck.html_tools.string_traceback()
@@ -1085,9 +1268,9 @@ def get_feedback(
                 ),
                 file=sys.stderr
             )
-            result = "missing"
+            result["status"] = "missing"
 
-    if result != "missing":
+    if result["status"] == "ok":
         # Polish up warnings/evaluation a tiny bit
         warnings = result.get("warnings", [])
         evaluation = result.get("evaluation", "not evaluated")
@@ -1100,6 +1283,26 @@ def get_feedback(
         result["warnings"] = warnings
         result["submitted"] = True
 
+    # Try to read log file if we couldn't get a report
+    if result["status"] == "missing":
+        if log_file is None:
+            result["log"] = "no submission was made"
+        else:
+            try:
+                if not os.path.exists(log_file):
+                    result["log"] = "missing"
+                else:
+                    with open(log_file, 'r') as fin:
+                        result["log"] = fin.read()
+            except Exception:
+                flask.flash("Error reading log file.")
+                tb = potluck.html_tools.string_traceback()
+                print(
+                    "Failed to read log file '{}':\n{}".format(log_file, tb),
+                    file=sys.stderr
+                )
+                result["log"] = "missing"
+
     return result
 
 
@@ -1119,8 +1322,7 @@ def get_feedback_html(
     expensive memory-wise over time) we hit the disk every time.
 
     Returns the string "missing" if the relevant feedback file does not
-    exist, or None if some kind of error occurs trying to access the
-    file.
+    exist, or if some kind of error occurs trying to access the file.
 
     Might encounter an error if the file is being written as we try to
     read it.
